import 'package:bird/bird.dart';
import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:bird_flutter/bird_flutter.dart';
import 'package:flutter/material.dart';
import 'package:neptune_cryptography_example/elements.dart';
import 'package:neptune_cryptography_example/widgets/blocs.dart';
import 'package:neptune_cryptography_example/widgets/input.dart';

class CiphersPage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return $$ >> (context) {
      return onListView() >> [
        ...[
          AnnotatedValue("aes_cbc", dart_ciphers.aes_cbc()),
          AnnotatedValue("aes_ctr", dart_ciphers.aes_ctr()),
        ].map(pwIvCipherTile),
        ...[
          AnnotatedValue("aes_ecb", dart_ciphers.aes_ecb()),
        ].map(pwCipherTile),
      ];
    };
  }
}

final CryptographyCiphers dart_ciphers = CryptographyCiphers.dart;

Widget pwIvCipherTile(
    AnnotatedValue<CryptoCipher<CryptoParametersPWAndIV>> val) =>
    $$ >> (context) =>
        ListTile(
          title: Text(val.name),
          onTap: () {
            showDialog<void>(
              context: context,
              builder: (_) =>
              $$ >> (context) {
                final bloc = $$$(() => CryptoCipherPWIVBloc(val.value));
                return AlertDialog(
                  title: Text(val.name),
                  content: singleChildScrollView() > onColumnMinCenterCenter() >> [
                    encodingWaved(
                      just(false),
                      bloc.sourceEncoding,
                    ),
                    signalInputWidget(
                      signal: bloc.sourceText,
                      label: just("Input"),
                    ),

                    verticalSpace(14.0),

                    encodingWaved(
                      just(false),
                      bloc.pwEncoding,
                    ),
                    signalInputWidget(
                      signal: bloc.pwText,
                      label: just("PW"),
                    ),

                    verticalSpace(14.0),

                    encodingWaved(
                      just(false),
                      bloc.ivEncoding,
                    ),
                    signalInputWidget(
                      signal: bloc.ivText,
                      label: just("IV"),
                    ),

                    verticalSpace(14.0),

                    encodingWaved(
                      bloc.isLoading.wave,
                      bloc.targetEncoding,
                    ),
                    signalInputWidget(
                      signal: bloc.targetText,
                      label: just("Output"),
                    ),
                  ],
                  actions: [
                    loadingIndicator(bloc.isLoading.wave),
                    calculate(bloc.isLoading.wave, () => bloc.convert(true))
                        > const Text("Encrypt"),
                    calculate(bloc.isLoading.wave, () => bloc.convert(false))
                        > const Text("Decrypt"),
                    closeButton,
                  ],
                );
              },
            );
          },
        );

Widget pwCipherTile(AnnotatedValue<CryptoCipher<CryptoParametersPW>> val) =>
    $$ >> (context) =>
        ListTile(
          title: Text(val.name),
          onTap: () {
            showDialog<void>(
              context: context,
              builder: (_) =>
              $$ >> (context) {
                final bloc = $$$(() => CryptoCipherPWBloc(val.value));
                return AlertDialog(
                  title: Text(val.name),
                  content: singleChildScrollView() > onColumnMinCenterCenter() >> [
                    encodingWaved(
                      just(false),
                      bloc.sourceEncoding,
                    ),
                    signalInputWidget(
                      signal: bloc.sourceText,
                      label: just("Input"),
                    ),

                    verticalSpace(14.0),

                    encodingWaved(
                      just(false),
                      bloc.pwEncoding,
                    ),
                    signalInputWidget(
                      signal: bloc.pwText,
                      label: just("PW"),
                    ),

                    verticalSpace(14.0),

                    encodingWaved(
                      bloc.isLoading.wave,
                      bloc.targetEncoding,
                    ),
                    signalInputWidget(
                      signal: bloc.targetText,
                      label: just("Output"),
                    ),
                  ],
                  actions: [
                    loadingIndicator(bloc.isLoading.wave),
                    calculate(bloc.isLoading.wave, () => bloc.convert(true))
                        > const Text("Encrypt"),
                    calculate(bloc.isLoading.wave, () => bloc.convert(false))
                        > const Text("Decrypt"),
                    closeButton,
                  ],
                );
              },
            );
          },
        );