import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:flutter/material.dart';
import 'package:neptune_cryptography_example/elements.dart';
import 'package:bird_flutter/bird_flutter.dart';
import 'package:neptune_cryptography_example/widgets/blocs.dart';
import 'package:neptune_cryptography_example/widgets/input.dart';

class HashesPage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return onListView() >> hashes.map(hashTile);
  }
}

const dart_hashes = CryptographyHashes.dart;

final List<AnnotatedValue<CryptographyHash>> hashes = [
  AnnotatedValue("keccak224", dart_hashes.keccak224()),
  AnnotatedValue("keccak256", dart_hashes.keccak256()),
  AnnotatedValue("keccak384", dart_hashes.keccak384()),
  AnnotatedValue("keccak512", dart_hashes.keccak512()),
  AnnotatedValue("md2", dart_hashes.md2()),
  AnnotatedValue("md4", dart_hashes.md4()),
  AnnotatedValue("md5", dart_hashes.md5()),
  AnnotatedValue("ripemd128", dart_hashes.ripemd128()),
  AnnotatedValue("ripemd160", dart_hashes.ripemd160()),
  AnnotatedValue("ripemd256", dart_hashes.ripemd256()),
  AnnotatedValue("ripemd320", dart_hashes.ripemd320()),
  AnnotatedValue("sha1", dart_hashes.sha1()),
  AnnotatedValue("sha224", dart_hashes.sha224()),
  AnnotatedValue("sha256", dart_hashes.sha256()),
  AnnotatedValue("sha384", dart_hashes.sha384()),
  AnnotatedValue("sha512", dart_hashes.sha512()),
  ...CryptographyHashes.allowedSha512tDigestSizes().map((size) {
    return AnnotatedValue("sha512/$size", dart_hashes.sha512t(size));
  }),
];

Widget hashTile(AnnotatedValue<CryptographyHash> hash) {
  return $$ >> (context) =>
      ListTile(
        title: Text(hash.name),
        onTap: () {
          showDialog<void>(
            context: context,
            builder: (_) =>
            $$ >> (context) {
              final bloc = $$$(() => DigestBloc(hash.value));
              return AlertDialog(
                title: Text(hash.name),
                content: singleChildScrollView() > onColumnMinCenterCenter() >> [
                  encodingWaved(
                    just(false),
                    bloc.sourceEncoding,
                  ),
                  signalInputWidget(
                    signal: bloc.sourceText,
                    label: just("Input"),
                  ),

                  verticalSpace(14.0),
                  encodingWaved(
                    bloc.isLoading.wave,
                    bloc.targetEncoding,
                  ),
                  signalInputWidget(
                    signal: bloc.targetText,
                    label: just("Output"),
                  ),
                ],
                actions: [
                  loadingIndicator(bloc.isLoading.wave),
                  calculate(bloc.isLoading.wave, bloc.convert)
                      > const Text("Convert"),
                  closeButton,
                ],
              );
            },
          );
        },
      );
}