import 'dart:typed_data';

import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:pointycastle/block/aes_fast.dart';
import 'package:pointycastle/block/modes/ecb.dart';
import 'package:pointycastle/pointycastle.dart';

class PointycastleCiphers implements CryptographyCiphers {
  const PointycastleCiphers();

  @override
  CryptoCipher<CryptoParametersPWAndIV> aes_cbc() {
    return _AnonymousCryptoCipher(
      (List<int> data, CryptoParametersPWAndIV param) => BlockModeCBC(
          AESFastEngine().blockSize,
          param.iv(),
          (a) => (AESFastEngine()..init(false, KeyParameter(param.pw())))
              .process(a)).crypt(Uint8List.fromList(data), false),
      (List<int> data, CryptoParametersPWAndIV param) => BlockModeCBC(
          AESFastEngine().blockSize,
          param.iv(),
          (a) => (AESFastEngine()..init(true, KeyParameter(param.pw())))
              .process(a)).crypt(Uint8List.fromList(data), true),
    );
  }

  @override
  CryptoCipher<CryptoParametersPWAndIV> aes_ctr() {
    return _AnonymousCryptoCipher(
      (List<int> data, CryptoParametersPWAndIV param) => FocusedCodec.raw(
              const CryptoEncodingByteList(),
              BlockModeCTR(
                  iv: param.iv(),
                  blockSize: AESFastEngine().blockSize,
                  encrypt: (a) => (AESFastEngine()
                        ..init(true, KeyParameter(param.pw())))
                      .process(a)).crypt(Uint8List.fromList(data)))
          .list,
      (List<int> data, CryptoParametersPWAndIV param) => FocusedCodec.raw(
              const CryptoEncodingByteList(),
              BlockModeCTR(
                  iv: param.iv(),
                  blockSize: AESFastEngine().blockSize,
                  encrypt: (a) => (AESFastEngine()
                        ..init(true, KeyParameter(param.pw())))
                      .process(a)).crypt(Uint8List.fromList(data)))
          .list,
    );
  }

  @override
  CryptoCipher<CryptoParametersPW> aes_ecb() {
    return _AnonymousCryptoCipher(
      (List<int> data, CryptoParametersPW param) => BlockModeECB(
        blockSize: AESFastEngine().blockSize,
        cipher: (ECBBlockCipher(AESFastEngine())
              ..init(false, KeyParameter(param.pw())))
            .process,
      ).crypt(Uint8List.fromList(data)),
      (List<int> data, CryptoParametersPW param) => BlockModeECB(
        blockSize: AESFastEngine().blockSize,
        cipher: (ECBBlockCipher(AESFastEngine())
              ..init(true, KeyParameter(param.pw())))
            .process,
      ).crypt(Uint8List.fromList(data)),
    );
  }
}

class _AnonymousCryptoCipher<T extends CryptoParameter>
    extends CryptoCipher<T> {
  final List<int> Function(List<int> data, T params) _decrypt;
  final List<int> Function(List<int> data, T params) _encrypt;

  const _AnonymousCryptoCipher(this._decrypt, this._encrypt);

  @override
  List<int> decrypt(List<int> data, T params) => _decrypt(data, params);

  @override
  List<int> encrypt(List<int> data, T params) => _encrypt(data, params);
}
