import 'dart:typed_data';

import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:pointycastle/export.dart';

class PointycastleCryptographyHMACs implements CryptographyHMACs {
  const PointycastleCryptographyHMACs();

  @override
  HMACCrypto md2() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(MD2Digest(), 16)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto md4() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(MD4Digest(), 64)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto md5() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(MD5Digest(), 64)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto ripemd128() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(RIPEMD128Digest(), 64)
              ..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto ripemd160() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(RIPEMD160Digest(), 64)
              ..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto sha1() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(SHA1Digest(), 64)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto sha224() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(SHA224Digest(), 64)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto sha256() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(SHA256Digest(), 64)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto sha384() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(SHA384Digest(), 128)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto sha512() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(SHA512Digest(), 128)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }

  @override
  HMACCrypto tiger() {
    return _AnonymousHMACCrypto((List<int> intList, List<int> pw) =>
        (HMac(TigerDigest(), 64)..init(KeyParameter(Uint8List.fromList(pw))))
            .process(Uint8List.fromList(intList)));
  }
}

class _AnonymousHMACCrypto extends HMACCrypto {
  final List<int> Function(List<int> intList, List<int> pw) digest;

  const _AnonymousHMACCrypto(this.digest);

  @override
  List<int> digestRaw(List<int> intList, List<int> pw) => digest(intList, pw);
}
