import 'dart:typed_data';

abstract class CryptoParameter {}

abstract class CryptoParametersPW implements CryptoParameter {
  static CryptoParametersPW create(Uint8List pw) => _CryptoParametersPW(pw);

  Uint8List pw();
}

class _CryptoParametersPW implements CryptoParametersPW {
  final Uint8List password;

  const _CryptoParametersPW(this.password) : assert(password != null);

  @override
  Uint8List pw() => password;
}

abstract class CryptoParametersPWAndIV implements CryptoParameter {
  static CryptoParametersPWAndIV create(Uint8List pw, Uint8List iv) =>
      _CryptoParametersPWAndIV(pw, iv);

  Uint8List pw();

  Uint8List iv();
}

class _CryptoParametersPWAndIV implements CryptoParametersPWAndIV {
  final Uint8List password;

  final Uint8List initVector;

  const _CryptoParametersPWAndIV(this.password, this.initVector)
      : assert(password != null),
        assert(initVector != null);

  @override
  Uint8List iv() => initVector;

  @override
  Uint8List pw() => password;
}
