import 'dart:typed_data';

import 'package:collection/collection.dart';
import 'package:bird_cryptography/bird_cryptography.dart';

class FocusedCodec {
  final List<int> rawListOfInts;

  final CryptographicDataEncoding encoding;

  Uint8List get list {
    return rawListOfInts == null ? null : Uint8List.fromList(rawListOfInts);
  }

  const FocusedCodec.raw(this.encoding, this.rawListOfInts)
      : assert(encoding != null),
        assert(rawListOfInts != null);

  factory FocusedCodec.encoded(CryptographicDataEncoding encoding,
      String raw) =>
      FocusedCodec.raw(encoding, encoding.decode(raw));

  factory FocusedCodec.hex(String raw) =>
      FocusedCodec.raw(const CryptoEncodingHex(), () {
        try {
          return const CryptoEncodingHex().decode(raw);
        } catch (e) {
          return null;
        }
      }());

  factory FocusedCodec.utf8(String raw) =>
      FocusedCodec.raw(const CryptoEncodingUTF8(), () {
        try {
          return const CryptoEncodingUTF8().decode(raw);
        } catch (e) {
          return null;
        }
      }());

  FocusedCodec updateType(CryptographicDataEncoding type) =>
      FocusedCodec.raw(type, rawListOfInts);

  String toTypedString() => printToTypeString(encoding);

  String printToTypeString(CryptographicDataEncoding dataType) =>
      rawListOfInts == null ? null : dataType.encode(rawListOfInts);

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
          other is FocusedCodec &&
              runtimeType == other.runtimeType &&
              const ListEquality<dynamic>()
                  .equals(rawListOfInts, other.rawListOfInts) &&
              encoding == other.encoding;

  @override
  int get hashCode =>
      const ListEquality<dynamic>().hash(rawListOfInts) ^ encoding.hashCode;
}
