import 'dart:convert';

import 'package:convert/convert.dart';
import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:test/test.dart';

void main() {
  group("$CryptographicDataEncoding", () {
    final _TestCryptographicDataEncoding _ = _TestCryptographicDataEncoding();

    test("equality", () {
      expect(e_base64, e_base64);
      expect(const CryptoEncodingBase64(), const CryptoEncodingBase64());
      expect(const CryptoEncodingBase64(), isNot(e_utf8));
    });
  });

  test("$CryptoEncodingBase64", () {
    const CryptographicDataEncoding encoding = CryptoEncodingBase64();
    nullTest(encoding);
    expect(
        const CryptoEncodingUTF8().encode(encoding.decode("dGVzdA==")), "test");
    expect(
        encoding.encode(const CryptoEncodingUTF8().decode("test")), "dGVzdA==");
  });

  test("$CryptoEncodingByteList", () {
    const CryptographicDataEncoding encoding = CryptoEncodingByteList();
    nullTest(encoding);
    expect(() => encoding.decode("raw"),
        throwsA(const TypeMatcher<FormatException>()));
    expect(() => encoding.decode("[1"),
        throwsA(const TypeMatcher<FormatException>()));
    expect(encoding.decode("[]"), <dynamic>[]);
    expect(encoding.decode("[1]"), [1]);
    expect(encoding.decode("[-5]"), null);
    expect(encoding.decode("[12341, 12343151]"), null);
    expect(encoding.encode([1]), "[1]");
    expect(() => encoding.encode([-5]),
        throwsA(const TypeMatcher<AssertionError>()));
    expect(encoding.encode([]), "[]");
    expect(encoding.encode([1, 2]), "[1, 2]");
    expect(() => encoding.encode([12341, 12343151]),
        throwsA(const TypeMatcher<AssertionError>()));
    expect(encoding.encode([255]), "[255]");
    expect(() => encoding.encode([256]),
        throwsA(const TypeMatcher<AssertionError>()));
  });

  test("$CryptoEncodingHex", () {
    const CryptographicDataEncoding encoding = CryptoEncodingHex();
    nullTest(encoding);
    expect(encoding.decode("74657374"), hex.decode("74657374"));
    expect(encoding.decode("0f"), [15]);
    expect(encoding.decode("0x74657374"), hex.decode("74657374"));
    expect(() => encoding.decode("0x7465z7374"),
        throwsA(const TypeMatcher<FormatException>()));
    expect(encoding.encode(hex.decode("74657374")), "74657374");
  });

  test("$CryptoEncodingIntList", () {
    const CryptographicDataEncoding encoding = CryptoEncodingIntList();
    nullTest(encoding);
    expect(() => encoding.decode("raw"),
        throwsA(const TypeMatcher<FormatException>()));
    expect(() => encoding.decode("[1"),
        throwsA(const TypeMatcher<FormatException>()));
    expect(encoding.decode("[]"), <dynamic>[]);
    expect(encoding.decode("[1]"), [1]);
    expect(encoding.decode("[-5]"), [-5]);
    expect(encoding.decode("[12341, 12343151]"), [12341, 12343151]);
    expect(encoding.encode([1]), "[1]");
    expect(encoding.encode([-5]), "[-5]");
    expect(encoding.encode([]), "[]");
    expect(encoding.encode([1, 2]), "[1, 2]");
    expect(encoding.encode([12341, 12343151]), "[12341, 12343151]");
  });

  test("$CryptoEncodingUnknown", () {
    const CryptographicDataEncoding encoding = CryptoEncodingUnknown();
    nullTest(encoding);
    expect(encoding.decode("test"), null);
    expect(encoding.encode([123]), null);
  });

  test("$CryptoEncodingUTF8", () {
    const CryptographicDataEncoding encoding = CryptoEncodingUTF8();
    nullTest(encoding);
    expect(encoding.decode("test"), hex.decode("74657374"));
    expect(encoding.encode(hex.decode("74657374")), "test");
  });
}

void nullTest(CryptographicDataEncoding encoding) {
  expect(encoding.decode(null), null);
  expect(encoding.encode(null), null);
}

class AnonymousConverter<S, T> extends Converter<S, T> {
  final T Function(S) _convert;

  const AnonymousConverter(this._convert);

  @override
  T convert(S input) => _convert(input);
}

class _TestCryptographicDataEncoding extends CryptographicDataEncoding {
  @override
  Converter<String, List<int>> get decoder => AnonymousConverter((raw) => null);

  @override
  Converter<List<int>, String> get encoder => AnonymousConverter((raw) => null);
}
