import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:test/test.dart';

void main() {
  group("kdf", () {
    const k = CryptographyKDF.dart;

    group("SHA256_PBKDF2", () {
      test("not null", () {
        // ignore: prefer_const_constructors
        expect(() => k.sha256_pbkdf2(i: 1, length: null),
            throwsA(const TypeMatcher<AssertionError>()));
        // ignore: prefer_const_constructors
        expect(() => k.sha256_pbkdf2(i: null, length: 4),
            throwsA(const TypeMatcher<AssertionError>()));
      });
      test("test", () {
        expect(
          digestData(
                  k.sha256_pbkdf2(
                    i: 1,
                    length: 32,
                  ),
                  FocusedCodec.utf8("salt"),
                  FocusedCodec.utf8("password"))
              .printToTypeString(const CryptoEncodingHex()),
          "120fb6cffcf8b32c43e7225256c4f837a86548c92ccc35480805987cb70be17b",
        );
        expect(
          digestData(
                  k.sha256_pbkdf2(
                    i: 4096,
                    length: 40,
                  ),
                  FocusedCodec.utf8("saltSALTsaltSALTsaltSALTsaltSALTsalt"),
                  FocusedCodec.utf8("passwordPASSWORDpassword"))
              .printToTypeString(const CryptoEncodingHex()),
          "348c89dbcbd32b2f32d814b8116e84cf2b17347ebc1800181c4e2a1fb8dd53e1c635518c7dac47e9",
        );
      });
    });

    group("SHA384_PBKDF2", () {
      test("not null", () {
        // ignore: prefer_const_constructors
        expect(() => k.sha384_pbkdf2(i: 1, length: null),
            throwsA(const TypeMatcher<AssertionError>()));
        // ignore: prefer_const_constructors
        expect(() => k.sha384_pbkdf2(i: null, length: 4),
            throwsA(const TypeMatcher<AssertionError>()));
      });
      test("test", () {
        expect(
          digestData(
                  k.sha384_pbkdf2(
                    i: 1,
                    length: 20,
                  ),
                  FocusedCodec.utf8("salt"),
                  FocusedCodec.utf8("password"))
              .printToTypeString(const CryptoEncodingHex()),
          "c0e14f06e49e32d73f9f52ddf1d0c5c719160923",
        );
      });
    });

    group("SHA512_PBKDF2", () {
      test("not null", () {
        // ignore: prefer_const_constructors
        expect(() => k.sha512_pbkdf2(i: 1, length: null),
            throwsA(const TypeMatcher<AssertionError>()));
        // ignore: prefer_const_constructors
        expect(() => k.sha512_pbkdf2(i: null, length: 4),
            throwsA(const TypeMatcher<AssertionError>()));
      });
      test("test", () {
        expect(
          digestData(
                  k.sha512_pbkdf2(
                    i: 1,
                    length: 20,
                  ),
                  FocusedCodec.utf8("salt"),
                  FocusedCodec.utf8("password"))
              .printToTypeString(const CryptoEncodingHex()),
          "867f70cf1ade02cff3752599a3a53dc4af34c7a6",
        );
      });
    });

    group("Scrypt", () {
      test("not null", () {
        // ignore: prefer_const_constructors
        expect(() => k.scrypt(n: null, r: 1, p: 1, length: 1),
            throwsA(const TypeMatcher<AssertionError>()));
        // ignore: prefer_const_constructors
        expect(() => k.scrypt(n: 1, r: null, p: 1, length: 1),
            throwsA(const TypeMatcher<AssertionError>()));
        // ignore: prefer_const_constructors
        expect(() => k.scrypt(n: 1, r: 1, p: null, length: 1),
            throwsA(const TypeMatcher<AssertionError>()));
        // ignore: prefer_const_constructors
        expect(() => k.scrypt(n: 1, r: 1, p: 1, length: null),
            throwsA(const TypeMatcher<AssertionError>()));
      });
      test("test", () {
        expect(
          digestData(
                  k.scrypt(
                    n: 16,
                    r: 1,
                    p: 1,
                    length: 64,
                  ),
                  FocusedCodec.hex(""),
                  FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "77d6576238657b203b19ca42c18a0497f16b4844e3074ae8dfdffa3fede21442fcd0069ded0948f8326a753a0fc81f17e8d3e0fb2e0d3628cf35e20c38d18906",
        );
        expect(
          digestData(
                  k.scrypt(
                    n: 1024,
                    r: 8,
                    p: 16,
                    length: 64,
                  ),
                  FocusedCodec.hex("4e61436c"),
                  FocusedCodec.hex("70617373776f7264"))
              .printToTypeString(const CryptoEncodingHex()),
          "fdbabe1c9d3472007856e7190d01e9fe7c6ad7cbc8237830e77376634b3731622eaf30d92e22a3886ff109279d9830dac727afb94a83ee6d8360cbdfa2cc0640",
        );
      });
    });
  });
}

FocusedCodec digestData(SimplePWDigest d, FocusedCodec data, FocusedCodec pw) =>
    FocusedCodec.raw(
      const CryptoEncodingIntList(),
      d.digestRaw(data.list, pw.list),
    );
