import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:test/test.dart';

void main() {
  group("$FocusedCodec", () {
    group("constructor", () {
      test("null", () {
        // ignore: prefer_const_constructors
        expect(() => FocusedCodec.raw(null, [2]),
            throwsA(const TypeMatcher<AssertionError>()));
        // ignore: prefer_const_constructors
        expect(() => FocusedCodec.raw(e_utf8, null),
            throwsA(const TypeMatcher<AssertionError>()));
      });
      test("all", () {
        expect(FocusedCodec.raw(e_utf8, [2]).encoding, e_utf8);
        expect(FocusedCodec.raw(e_utf8, [2]).list, [2]);
      });
      test("decoded", () {
        expect(FocusedCodec.raw(e_utf8, [2]).encoding, e_utf8);
        expect(FocusedCodec.raw(e_utf8, [2]).list, [2]);
      });
      test("encoded", () {
        expect(FocusedCodec.encoded(e_utf8, "abc").encoding, e_utf8);
        expect(FocusedCodec.encoded(e_utf8, "abc").list, e_utf8.decode("abc"));
      });
      test("hex", () {
        expect(FocusedCodec.hex("0f").encoding, e_hex);
        expect(FocusedCodec.hex("0f").list, [15]);
      });
      test("utf8", () {
        expect(FocusedCodec.utf8("test").encoding, e_utf8);
        expect(FocusedCodec.utf8("test").list, e_utf8.decode("test"));
      });
    });

    test("rawListOfIntsOrNull", () {
      expect(FocusedCodec.hex("0f").list, [15]);
      expect(FocusedCodec.hex("0f").list, [15]);
      expect(() => FocusedCodec.hex("0fg").list,
          throwsA(const TypeMatcher<AssertionError>()));
      expect(() => FocusedCodec.hex("0fg").rawListOfInts,
          throwsA(const TypeMatcher<AssertionError>()));
    });

    test("updateType", () {
      expect(FocusedCodec.hex("0a").updateType(e_utf8),
          FocusedCodec.raw(e_utf8, [10]));
    });

    test("toTypedString", () {
      expect(FocusedCodec.hex("0a").toTypedString(),
          e_hex.encode(e_hex.decode("0a")));
    });

    test("printToTypeString", () {
      expect(FocusedCodec.hex("0f").printToTypeString(e_bytelist), "[15]");
    });

    test("hashCode", () {
      expect(
        FocusedCodec.hex("0f").hashCode,
        FocusedCodec.hex("0f").hashCode,
      );
    });

    test("==", () {
      final a = FocusedCodec.hex("0f");
      expect(a, a);
      expect(a, FocusedCodec.hex("0f"));
      expect(FocusedCodec.hex("0f"), FocusedCodec.hex("0f"));
    });

    test("!=", () {
      expect(FocusedCodec.hex("0a"), isNot(FocusedCodec.hex("0f")));
      expect(FocusedCodec.utf8("0a"), isNot(FocusedCodec.hex("0f")));
    });
  });
}
