import 'dart:typed_data';

import 'package:test/test.dart';
import 'package:bird_cryptography/bird_cryptography.dart';

void main() {
  group("util.dart", () {
    test("pad", () {
      expect(pad([1, 2, 3], PaddingLocation.right, 5), [1, 2, 3, 0, 0]);
      expect(pad([1, 2, 3], PaddingLocation.left, 5), [0, 0, 1, 2, 3]);
      expect(pad([1, 2, 3], PaddingLocation.right, 5, symbol: 40),
          [1, 2, 3, 40, 40]);
      expect(pad([1, 2, 3], PaddingLocation.left, 5, symbol: 40),
          [40, 40, 1, 2, 3]);
      expect(
          pad([1, 2, 3, 4, 5, 6], PaddingLocation.right, 5), [2, 3, 4, 5, 6]);
      expect(pad([1, 2, 3, 4, 5, 6], PaddingLocation.left, 5), [1, 2, 3, 4, 5]);
    });

    test("padBytes", () {
      expect(padbytes(Uint8List.fromList([1, 2, 3]), PaddingLocation.right, 5),
          [1, 2, 3, 0, 0]);
      expect(padbytes(Uint8List.fromList([1, 2, 3]), PaddingLocation.left, 5),
          [0, 0, 1, 2, 3]);
      expect(
          padbytes(Uint8List.fromList([1, 2, 3]), PaddingLocation.right, 5,
              symbol: 40),
          [1, 2, 3, 40, 40]);
      expect(
          padbytes(Uint8List.fromList([1, 2, 3]), PaddingLocation.left, 5,
              symbol: 40),
          [40, 40, 1, 2, 3]);
      expect(
          padbytes(
              Uint8List.fromList([1, 2, 3, 4, 5, 6]), PaddingLocation.right, 5),
          [2, 3, 4, 5, 6]);
      expect(
          padbytes(
              Uint8List.fromList([1, 2, 3, 4, 5, 6]), PaddingLocation.left, 5),
          [1, 2, 3, 4, 5]);
    });

    test("xor", () {
      expect(xor([4, 1], [8, 2], 3, PaddingLocation.right), [12, 3, 0]);
      expect(xor([8, 2], [4], 3, PaddingLocation.right), [12, 2, 0]);
      expect(xor([128, 12], [64, 24], 3, PaddingLocation.left, symbol: 50),
          [50, 192, 20]);
      expect(xor([128, 12], [99, 64, 24], 4, PaddingLocation.left, symbol: 50),
          [50, 99, 192, 20]);
    });

    test("xorbytes", () {
      expect(
        xorbytes(Uint8List.fromList([4, 1]), Uint8List.fromList([8, 2]), 3,
            PaddingLocation.right),
        Uint8List.fromList([12, 3, 0]),
      );
      expect(
        xor(Uint8List.fromList([4]), Uint8List.fromList([8, 2]), 3,
            PaddingLocation.right),
        Uint8List.fromList([12, 2, 0]),
      );
      expect(
        xorbytes(Uint8List.fromList([128, 12]), Uint8List.fromList([64, 24]), 3,
            PaddingLocation.left,
            symbol: 50),
        Uint8List.fromList([50, 192, 20]),
      );
      expect(
        xorbytes(Uint8List.fromList([128, 12]),
            Uint8List.fromList([99, 64, 24]), 4, PaddingLocation.left,
            symbol: 50),
        Uint8List.fromList([50, 99, 192, 20]),
      );
    });

    test("$PaddingLocation", () {
      /// Nothing to test
      const PaddingLocation _ = PaddingLocation.left;
    });

    test("divideIntoBlocks", () {
      expect(
        divideIntoBlocks(Uint8List.fromList([1, 2, 3, 4, 5]), 2),
        [
          Uint8List.fromList([1, 2]),
          Uint8List.fromList([3, 4]),
          Uint8List.fromList([5])
        ],
      );
    });
  });
}
