import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:test/test.dart';

void main() {
  group("hmacs", () {
    final key = FocusedCodec.utf8(
        "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
    final msg = FocusedCodec.utf8(
        "54657374205573696e67204c6172676572205468616e20426c6f636b2d53697a65204b6579202d2048617368204b6579204669727374");

    const h = CryptographyHMACs.dart;

    test("HMAC_MD2", () {
      expect(
        digestData(h.md2(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "62018c9cd0d5feea0b11193380ffaad1",
      );
    });

    test("HMAC_MD4", () {
      expect(
        digestData(h.md4(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "adbd5c2fef6795b02636b79b3e6fac55",
      );
    });

    test("HMAC_MD5", () {
      expect(
        digestData(h.md5(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "0ad72fbe62a490ef1150f740933730ec",
      );
    });

    test("HMAC_RIPEMD128", () {
      expect(
        digestData(h.ripemd128(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "082b8a65989376f1aa178cd5eb16f926",
      );
    });

    test("HMAC_RIPEMD160", () {
      expect(
        digestData(h.ripemd160(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "1aa4907801a5eb09161d20409deb69c7e027beed",
      );
    });

    test("HMAC_SHA1", () {
      expect(
        digestData(h.sha1(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "3af5f8ef46d8fad7a4a6eab636765beccb4b3184",
      );
    });

    test("HMAC_SHA224", () {
      expect(
        digestData(h.sha224(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "3ce22b8f9d85051054a4fde3800451ae443069fd41bd4e1a0859a33b",
      );
    });

    test("HMAC_SHA256", () {
      expect(
        digestData(h.sha256(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "4b84e0533ecd723db32d2e24377bfbad151d4fd61879cdbab7b9549caa48eb10",
      );
    });

    test("HMAC_SHA384", () {
      expect(
        digestData(h.sha384(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "b9dffb5ade5e9168e098d987316ac300331f1cacd3a7e447e2561eb1c375967155b581bf77b63e37f0024ce1376d7c67",
      );
    });

    test("HMAC_SHA512", () {
      expect(
        digestData(h.sha512(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "5a32d5edf8d397c5f68f9025781cef04c870f9578a3d7719433a4b7ec19ecddddf4f13a7c03e794bc4202dd1aa34cd4e4655429f8d4ebdce0f6463bc7d7b9b31",
      );
    });

    test("HMAC_Tiger", () {
      expect(
        digestData(h.tiger(), msg, key)
            .printToTypeString(const CryptoEncodingHex()),
        "eb76406d12bfd5c825c3a99874897d4979b851f7cdc39764",
      );
    });
  });
}

FocusedCodec digestData(SimplePWDigest d, FocusedCodec data, FocusedCodec pw) =>
    FocusedCodec.raw(
      const CryptoEncodingIntList(),
      d.digestRaw(data.list, pw.list),
    );
